package weaver.hrm.cachecenter.util;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;
import java.util.StringTokenizer;
import java.util.regex.Pattern;

import org.apache.commons.lang3.StringUtils;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import weaver.conn.RecordSet;
import weaver.hrm.User;
import weaver.hrm.cachecenter.bean.LoadComInfo;
import weaver.hrm.cachecenter.bean.SuperDepartmentComInfo;
import weaver.hrm.cachecenter.bean.SuperSubCompanyComInfo;
import weaver.hrm.cachecenter.entry.ConstStyle;
import weaver.hrm.cachecenter.entry.HrmTypeStyle;
import weaver.hrm.cachecenter.entry.OrgUpOrDownTypeStyle;
import weaver.hrm.cachecenter.entry.OrgtypeInfo;
import weaver.hrm.company.DepartmentComInfo;
import weaver.hrm.company.SubCompanyComInfo;
import weaver.hrm.resource.ResourceComInfo;

public class ObjtypeParser {

	private static final Logger log = LoggerFactory.getLogger("parallel");
	
	private ResourceComInfo resourceInfo  ;
	private DepartmentComInfo deptInfo  ;
	private SuperDepartmentComInfo superDeptInfo ;
	private SubCompanyComInfo subcompanyInfo ; 
	private SuperSubCompanyComInfo superSubcompanyInfo ;
	
	
	private ObjtypeParser(){
		try {
			resourceInfo = new ResourceComInfo() ;
			deptInfo = new DepartmentComInfo() ;
			superDeptInfo = LoadComInfo.getInstance(SuperDepartmentComInfo.class) ;
			subcompanyInfo = new SubCompanyComInfo() ;
			superSubcompanyInfo = LoadComInfo.getInstance(SuperSubCompanyComInfo.class) ;
		} catch (Exception e) {
		}
	} ;
	
	public static ObjtypeParser getInstance(){
		return new ObjtypeParser()  ;
	}
	
	
	
	
	/**
	 * 批量解析，以分号分隔
	 * @param desc
	 * @param user
	 * @return
	 * @throws Exception
	 */
	public List<OrgtypeInfo> parse(String desc,User user) {
		
		List<OrgtypeInfo> list = new ArrayList<OrgtypeInfo>() ;
		if(StringUtils.isBlank(desc)) return list ;
		for (String one : desc.split(";")) {
		
			OrgtypeInfo typeInfo = parseOne(one, user) ;
			if(typeInfo != null) list.add(typeInfo) ;
		}
		
		return list ;
	}
	
	private static final Pattern SPLIT_PATTERN = Pattern.compile("[|]") ;
	
	/**
	 * 解析单个描述符
	 * @param one
	 * @param user
	 * @return
	 */
	public OrgtypeInfo parseOne(String one,User user){
		String[] infos = SPLIT_PATTERN.split(one,-1) ;
		if(infos.length != 8) return null ;
		String objid = infos[0] ;
		String objtype =  infos[1] ;
		String rolelevel =  infos[2] ;
		String joblevel =  infos[3] ;
		String jobobjid =  infos[4] ;
		String seclevel =  infos[5] ;
		String seclevelMax =  infos[6] ;
		String relationship =  infos[7] ;

		if ("16".equals(objtype)){

			int uid = user.getUID();

			log.info("【uid】==========="+uid);

			String sql = "select id from hrmdepartment where supdepid=" +
					"(select d.supdepid from HrmResource r join hrmdepartment d on r.departmentid=d.id where r.id='" + uid + "')" +
					"and subcompanyid1 = (select subcompanyid1 from HrmResource where id='" + uid + "')";

			log.info("【sql】==========="+sql);

			RecordSet recordSet = new RecordSet();
			boolean flag = recordSet.executeQuery(sql);

			String uids = "";
			if(flag){
				while (recordSet.next()) {
					uids += recordSet.getString("id") + ",";
				}
				if (uids.endsWith(",")){
					objid = uids.substring(0,uids.length()-1);
				}
			}

			log.info("【objid】==========="+objid);

			objtype="10";
		}
		
		OrgtypeInfo typeInfo = new OrgtypeInfo() ;
		HrmTypeStyle style = HrmTypeStyle.findById(objtype) ;
		
		typeInfo.setId(objid) ;
		typeInfo.setType(style) ;
		typeInfo.setRolelevel(rolelevel) ;
		typeInfo.setJoblevel(joblevel) ;
		typeInfo.setJobobjid(jobobjid) ;
		typeInfo.setRolelevel(rolelevel) ;
		typeInfo.setSeclevelBegin(seclevel) ;
		typeInfo.setSeclevelEnd(seclevelMax) ;
		typeInfo.setIn(StringUtils.isBlank(relationship) ||"0".equals(relationship)) ;
		
		switch (style) {
		case SELF_DEPARTMENT:
			resetSelfDepartment(typeInfo,user);
			break;
		case SELF_UPER_DEPARTMENT:
			resetSelfUpperDepartment(typeInfo,user);
			break;
		case SELF_UPER_ALL_DEPARTMENT:
			resetSelfUperAllDepartment(typeInfo,user) ;
			break;
		case SELF_DOWN_DEPARTMENT:
			resetSelfDownDepartment(typeInfo,user) ;
			break;
		case SELF_DOWN_ALL_DEPARTMENT:
			resetSelfDownAllDepartment(typeInfo,user) ;
			break;
		case SELF_SUBCOMPANY:
			resetSelfSubcompany(typeInfo,user) ;
			break;
		case SELF_UPER_SUBCOMPANY:
			resetSelfUperSubcompany(typeInfo,user) ;
			break;
		case SELF_UPER_ALL_SUBCOMPANY:
			resetSelfUperAllSubCompany(typeInfo,user) ;
			break;
		case SELF_DOWN_SUBCOMPANY:
			resetSelfDownSubcompany(typeInfo,user) ;
			break;
		case SELF_DOWN_ALL_SUBCOMPANY:
			resetSelfDownAllSubcompany(typeInfo,user);
			break;
		default:
			break;
		}

		if(typeInfo.getId() == null) typeInfo.setId(StringUtils.EMPTY);
		
		return typeInfo ;
	}
	
	// 所有下级分部
	private void resetSelfDownAllSubcompany(OrgtypeInfo typeInfo, User user) {
		String id = resourceInfo.getSubCompanyID(Integer.toString(user.getUID())) ;
		
		//Set<String> set = new HashSet<String>() ;
		//getLowerSubIdByCircle(id,set) ;
		//String allLower = ListUtil.loopIds(set);
		
		typeInfo.setType(HrmTypeStyle.SUBCOMPANY) ;
		typeInfo.setId(id) ;
		typeInfo.setOrgUpOrDownType(OrgUpOrDownTypeStyle.ORG_ALL_DOWN) ;
		
	}
	
//	private void getLowerSubIdByCircle(String id, Set<String> set) {
//		if(StringUtils.isBlank(id)) return ;
//		for(String superId : id.split(",")){
//			set.add(superId) ;
//			String lowerIds = superSubcompanyInfo.getDownIdsBySuperId(superId) ;
//			getLowerSubIdByCircle(lowerIds,set);
//		}
//	}

	// 直接下级分部
	private void resetSelfDownSubcompany(OrgtypeInfo typeInfo, User user) {
		String id = resourceInfo.getSubCompanyID(Integer.toString(user.getUID())) ;
		String lowerId = superSubcompanyInfo.getDownIdsBySuperId(id) ;
		typeInfo.setType(HrmTypeStyle.SUBCOMPANY) ;
		typeInfo.setId(lowerId) ;
	}

	// 所有上级分部
	private void resetSelfUperAllSubCompany(OrgtypeInfo typeInfo, User user) {
		String id = resourceInfo.getSubCompanyID(Integer.toString(user.getUID())) ;
		//Set<String> set = new HashSet<String>() ;
		//getSuperSubIdByCircle(id,set) ;
		
		//String allSuper = ListUtil.loopIds(set);
		
		typeInfo.setType(HrmTypeStyle.SUBCOMPANY) ;
		typeInfo.setId(id) ;
		typeInfo.setOrgUpOrDownType(OrgUpOrDownTypeStyle.ORG_ALL_UP) ;
	}
	
	
	
	
	
	
//	private void getSuperSubIdByCircle(String id,Set<String> set){
//		String superId = subcompanyInfo.getSupsubcomid(id) ;
//		if(!StringUtils.isBlank(superId) && !"0".equals(superId)){
//			set.add(superId) ;
//			getSuperSubIdByCircle(superId,set) ;
//		}
//	}

	private void resetSelfUperSubcompany(OrgtypeInfo typeInfo, User user) {
		String id = resourceInfo.getSubCompanyID(Integer.toString(user.getUID())) ;
		String superId = subcompanyInfo.getSupsubcomid(id) ;
		typeInfo.setType(HrmTypeStyle.SUBCOMPANY) ;
		typeInfo.setId(superId) ;
	}

	private void resetSelfSubcompany(OrgtypeInfo typeInfo, User user) {
		String id = resourceInfo.getSubCompanyID(Integer.toString(user.getUID())) ;
		typeInfo.setType(HrmTypeStyle.SUBCOMPANY) ;
		typeInfo.setId(id) ;
	}
	
	/**
	 * 含所有下级
	 * @param typeInfo
	 * @param user
	 */
	private void resetSelfDownAllDepartment(OrgtypeInfo typeInfo, User user) {
		String deptid = resourceInfo.getDepartmentID(Integer.toString(user.getUID())) ;
		//String underDeptids = superDeptInfo.getDepartIdsBySuperId(deptid) ;
		
		//Set<String> set = new HashSet<String>() ;
		//getLowerIdsByCicle(underDeptids,set);
		
		typeInfo.setType(HrmTypeStyle.DEPARTMENT) ;
		//typeInfo.setId(ListUtil.loopIds(set)) ;
		typeInfo.setId(deptid) ;
		typeInfo.setOrgUpOrDownType(OrgUpOrDownTypeStyle.ORG_ALL_DOWN) ;
	}

	private void resetSelfDownDepartment(OrgtypeInfo typeInfo, User user) {
		String deptid = resourceInfo.getDepartmentID(Integer.toString(user.getUID())) ;
		String lowerIds = superDeptInfo.getDepartIdsBySuperId(deptid) ;
		typeInfo.setType(HrmTypeStyle.DEPARTMENT) ;
		typeInfo.setId(lowerIds) ;
	}
	/**
	 * 含所有上级
	 * @param typeInfo
	 * @param user
	 */
	private void resetSelfUperAllDepartment(OrgtypeInfo typeInfo, User user) {
		String deptid = resourceInfo.getDepartmentID(Integer.toString(user.getUID())) ;
		//Set<String> set = new HashSet<String>() ;
		//getSuperDeptIdByCircle(deptid,set) ;
		
		typeInfo.setType(HrmTypeStyle.DEPARTMENT) ;
		//typeInfo.setId(ListUtil.loopIds(set)) ;
		typeInfo.setId(deptid) ;
		typeInfo.setOrgUpOrDownType(OrgUpOrDownTypeStyle.ORG_ALL_UP) ;
		
	}
	
//	//获取所有下级
//	private void getLowerIdsByCicle(String ids,Set<String> set){
//		if(StringUtils.isBlank(ids)) return ;
//		for(String superId : ids.split(",")){
//			set.add(superId) ;
//			String lowerIds = superDeptInfo.getDepartIdsBySuperId(superId) ;
//			getLowerIdsByCicle(lowerIds,set);
//		}
//	}
	
//	// 获取全部上级
//	private void getSuperDeptIdByCircle(String id,Set<String> set){
//		String superId = deptInfo.getDepartmentsupdepid(id) ;
//		if(!StringUtils.isBlank(superId) && !"0".equals(superId)){
//			set.add(superId) ;
//			getSuperDeptIdByCircle(superId,set) ;
//		}
//	}

	private void resetSelfUpperDepartment(OrgtypeInfo typeInfo, User user) {
		String deptid = resourceInfo.getDepartmentID(Integer.toString(user.getUID())) ;
		String superId = deptInfo.getDepartmentsupdepid(deptid) ;
		typeInfo.setType(HrmTypeStyle.DEPARTMENT) ;
		typeInfo.setId(superId) ;
	}

	private void resetSelfDepartment(OrgtypeInfo info,User user){
		String deptid = resourceInfo.getDepartmentID(Integer.toString(user.getUID())) ;
		info.setType(HrmTypeStyle.DEPARTMENT) ;
		info.setId(deptid) ;
	}
}
