package weaver.workflow.workflow;

import com.api.browser.bean.SearchConditionGroup;
import com.api.browser.bean.SearchConditionItem;
import com.api.browser.bean.SearchConditionOption;
import com.api.browser.util.ConditionFactory;
import com.api.browser.util.ConditionType;
import weaver.conn.RecordSet;
import weaver.general.BaseBean;
import weaver.general.TimeUtil;
import weaver.general.Util;
import weaver.hrm.User;
import weaver.hrm.cachecenter.bean.WorkflowAdvancedComInfo;
import weaver.hrm.cachecenter.bean.WorkflowRecSimpleComInfo;
import weaver.systeminfo.SystemEnv;

import java.util.*;

/**
 * 转发、意见征询、转办 接收人范围设置
 * @author Administrator
 *
 */
public class WfFwLimitUtil extends BaseBean{
	
	private boolean debug = true;

	public String generateID() {
		UUID uuid  =  UUID.randomUUID(); 
		String id = uuid.toString().replaceAll("-", "");
		return id;
	}

	/**
	 * 供人力资源组调用，true需要范围控制。
	 * @param forwardflag //1:转发 2：意见征询 3：转办 4：传阅 5:征询转办
	 * @return
	 */
	public static boolean checkForwardflag(String forwardflag){
		if("1".equals(forwardflag) || "2".equals(forwardflag) || "3".equals(forwardflag)||
				"4".equals(forwardflag) || "5".equals(forwardflag) || "6".equals(forwardflag)){
			return true;
		}
		return false;
	}
	
	/**
	 * 是否开启了 限制接收人可选范围
	 * @param wfid
	 * @param nodeid
	 * @param fwtype 1转发 2意见征询 3转办
	 * @return 1 ：已开启
	 */
	public boolean isOpenLimit(int wfid,int nodeid,String fwtype){
		boolean flag = false;
		RecordSet rs = new RecordSet();
		String sql = "select isopen from workflow_FwLimitSet where isopen='1' and fwtype='"+fwtype+"' and nodeid="+nodeid+" and wfid="+wfid;
		rs.execute(sql);
		if(rs.next()){
			flag = true;
		}
		if(debug)writeLog(">>>>>>>>>isOpenLimit>>>sql="+sql);
		return flag;
	}

	public boolean choiceOperator(int wfid,int nodeid,String fwtype){
		boolean flag = false;
		RecordSet rs = new RecordSet();
		String sql = "select isopen from workflow_FwLimitSet where isopen='1' and fwtype='"+fwtype+"' and choiceoperator='1' and nodeid="+nodeid+" and wfid="+wfid;
		rs.execute(sql);
		if(rs.next()){
			flag = true;
		}
		if(debug)writeLog(">>>>>>>>>choiceOperator>>>sql="+sql);
		return flag;
	}
	
	/**
	 * 同步到所有节点
	 * @param wfid
	 * @param nodeid
	 * @param fwtype 1转发 2意见征询 3转办
	 * @param user
	 */
	public void syncAllNode(int wfid,int nodeid,String fwtype,User user){
		RecordSet rs = new RecordSet();
		String copy2nodeids = "";
		String sql = "select distinct f.nodeid from workflow_flownode f,workflow_nodebase n where (n.isfreenode is null or n.isfreenode !='1') and f.nodeid=n.id and f.nodeid !="+nodeid+" and f.workflowid="+wfid;
		rs.execute(sql);
		while(rs.next()){
			int tmpnodeid = rs.getInt("nodeid");
			copy2nodeids += ","+tmpnodeid;
		}
		if(!"".equals(copy2nodeids)){
			copy2nodeids = copy2nodeids.substring(1);
		}
		syncAllNode(wfid, nodeid, fwtype, user, copy2nodeids);
	}
	
	/**
	 * 同步到其他节点
	 * @param wfid
	 * @param nodeid
	 * @param fwtype  1转发 2意见征询 3转办
	 * @param user
	 * @param copy2nodeids  同步到目标节点 格式：1,2,3
	 */
	public void syncAllNode(int wfid,int nodeid,String fwtype,User user,String copy2nodeids){
		int id = 0;
		String isopen = "";
		String modetype = "";
		String choiceoperator = "";
		RecordSet rs = new RecordSet();
		String sql = "select * from workflow_FwLimitSet where fwtype='"+fwtype+"' and nodeid="+nodeid+" and wfid="+wfid+" ";
		rs.execute(sql);
		if(rs.next()){
			id = rs.getInt("id");
			isopen = Util.null2String(rs.getString("isopen"));
			modetype = Util.null2String(rs.getString("modetype"));
			choiceoperator = Util.null2o(rs.getString("choiceoperator"));
		}
		
		Map<String, String> param = new HashMap<String, String>();
		param.put("id", id+"");
		param.put("isopen", isopen);
		param.put("modetype", modetype);
		param.put("choiceoperator",choiceoperator);
		
		if(id>0){
			//if("1".equals(modetype)){ //高级模式
			syncNodeAdvanced(param, wfid, nodeid, fwtype, user,copy2nodeids);
			//}else{ //简单模式
			syncNodeSimple(param, wfid, nodeid, fwtype, user,copy2nodeids);
			//}
		}
	}
	
	
	/**
	 * 供 另存为新版本 使用
	 * 转发、意见征询、转办 范围设置
	 * @param newwfid
	 * @param newnodeid
	 * @param oldwfid
	 * @param oldnodeid
	 * @param fwtype
	 * @param user
	 */
	public void saveAsNewNode(int newwfid,int newnodeid,int oldwfid,int oldnodeid,String fwtype,int userid){
		int id = 0;
		String isopen = "";
		String modetype = "";
		String choiceoperator = "";
		String copy2nodeids = newnodeid+"";
		RecordSet rs = new RecordSet();
		String sql = "select * from workflow_FwLimitSet where fwtype='"+fwtype+"' and nodeid="+oldnodeid+" and wfid="+oldwfid+" ";
		rs.execute(sql);
		if(rs.next()){
			id = rs.getInt("id");
			isopen = Util.null2String(rs.getString("isopen"));
			modetype = Util.null2String(rs.getString("modetype"));
			choiceoperator = Util.null2String(rs.getString("choiceoperator"));
		}
		try {
			User user = new User();
			user.setUid(userid);
			
			Map<String, String> param = new HashMap<String, String>();
			param.put("id", id+"");
			param.put("isopen", isopen);
			param.put("modetype", modetype);
			param.put("choiceoperator",choiceoperator);
			
			param.put("oldwfid", oldwfid+"");
			param.put("oldnodeid", oldnodeid+"");
			param.put("fromNewnode", "1"); //节点是新创建的

			if(id > 0){
				//if("1".equals(modetype)){ //高级模式
				syncNodeAdvanced(param, newwfid, newnodeid, fwtype, user,copy2nodeids);
				//}else{ //简单模式
				syncNodeSimple(param, newwfid, newnodeid, fwtype, user,copy2nodeids);
				//}
			}
		} catch (Exception e) {
			writeLog(e);
		}
	}
	
	
	/**
	 * 高级模式同步
	 * @param param
	 * @param wfid
	 * @param nodeid
	 * @param fwtype 1转发 2意见征询 3转办
	 * @param user
	 * @param copy2nodeids 同步到目标节点 格式：1,2,3
	 */
	public void syncNodeAdvanced(Map<String, String> param,int wfid,int nodeid,String fwtype,User user,String copy2nodeids){
		String currentdate = TimeUtil.getCurrentDateString();
		String currenttime = TimeUtil.getOnlyCurrentTimeString();
		RecordSet rs = new RecordSet();
		RecordSet rs1 = new RecordSet();
		String sql = "";
		String sqlwhere = "";
		String isopen = Util.null2String(param.get("isopen"));
		String modetype= Util.null2String(param.get("modetype"));
		String choiceoperator = Util.null2String(param.get("choiceoperator"));
		
		int oldwfid = Util.getIntValue(param.get("oldwfid"),-1);
		int oldnodeid = Util.getIntValue(param.get("oldnodeid"),-1);
		String fromNewnode = Util.null2String(param.get("fromNewnode")); 
		
		if(debug)writeLog(">>>>>>syncNodeAdvanced>>>>0>>oldwfid="+oldwfid+" oldnodeid="+oldnodeid+" fromNewnode="+fromNewnode);		
		if(oldwfid<=0 && oldnodeid<=0){
			oldwfid = wfid;
			oldnodeid = nodeid;
		}		
		if(debug)writeLog(">>>>>>syncNodeAdvanced>>>>1>>oldwfid="+oldwfid+" oldnodeid="+oldnodeid);
		
		if(!"".equals(copy2nodeids)){
			sqlwhere = " and nodeid in ("+copy2nodeids+") ";
		}else{
			sqlwhere = " and 1=2 ";
		}
		
		if(debug)writeLog(">>>>>>syncNodeAdvanced>>>>>>sqlwhere="+sqlwhere);
		
		if(!"1".equals(fromNewnode)){ //新创建的节点 没有脏数据，不需要清除
			//操作者
			sql = "delete from workflow_FwOperAdvanced where mainid in (select id from workflow_FwAdvanced where fwtype='"+fwtype+"' "+sqlwhere+" and wfid="+wfid+" )";
			rs.execute(sql);
			//接收人范围
			sql = "delete from workflow_FwRecAdvanced where mainid in (select id from workflow_FwAdvanced where fwtype='"+fwtype+"' "+sqlwhere+" and wfid="+wfid+" )";
			rs.execute(sql);
			//操作者-接收人范围 组
			sql = "delete from workflow_FwAdvanced where fwtype='"+fwtype+"' "+sqlwhere+" and wfid="+wfid+" ";
			rs.execute(sql);
			//是否限制接收人可选范围设置。高级模式下该表谨慎删，高级模式与简单模式数据分开存放，但简单模式用到了该表的ID，如果直接删 会留下简单模式的脏数据
			//sql = "delete from workflow_FwLimitSet where fwtype='"+fwtype+"' "+" and wfid="+wfid+" ";
		}
		
		if(debug)writeLog(sql);
		
		List<Integer> ids = new ArrayList<Integer>();
		sql = "select id from workflow_FwAdvanced where fwtype='"+fwtype+"' and nodeid="+oldnodeid+" and wfid="+oldwfid+" order by id ";
		rs.execute(sql);
		while(rs.next()){
			ids.add(rs.getInt("id"));
		}
		
		if(debug)writeLog(sql);
		
		List<Integer> nodeids = new ArrayList<Integer>();
		List<String> oldnodeids = Util.splitString2List(copy2nodeids, ",");
		for (int i = 0; i < oldnodeids.size(); i++) {
			int tmpnodeid = Util.getIntValue(oldnodeids.get(i));
			if(tmpnodeid<=0)continue;
			nodeids.add(tmpnodeid);
		}
		
		if(debug)writeLog(">>>>>>syncNodeAdvanced>>>>>>nodeids="+nodeids);
		
		for (int k = 0; k < nodeids.size(); k++) {
			int tmpnodeid = nodeids.get(k);
			
			sql = "select id from workflow_FwLimitSet where fwtype='"+fwtype+"' and nodeid="+tmpnodeid+" and wfid="+wfid+" ";
			rs1.execute(sql);
			if(rs1.next()){
				sql = "update workflow_FwLimitSet set modetype=?,isopen=?,choiceoperator=?,lastoperator=?,lastoperatedate=?,lastoperatetime=? where fwtype='"+fwtype+"' and nodeid="+tmpnodeid+" and wfid="+wfid+" ";
				rs1.executeUpdate(sql, modetype,isopen,choiceoperator,user.getUID(),currentdate,currenttime);
			}else{
				sql = "insert into workflow_FwLimitSet(wfid,nodeid,fwtype,modetype,isopen,choiceoperator,lastoperator,lastoperatedate,lastoperatetime) values (?,?,?,?,?,?,?,?,?) ";
				rs1.executeUpdate(sql,wfid,tmpnodeid,fwtype, modetype,isopen,choiceoperator,user.getUID(),currentdate,currenttime);
			}
			
			for (int i = 0; i < ids.size(); i++) {
				int currid = ids.get(i);
				
				int mainid = 0;
				String uuid = generateID();
	    		sql = "insert into workflow_FwAdvanced (wfid,nodeid,fwtype,uuid) values ("+wfid+","+tmpnodeid+",'"+fwtype+"','"+uuid+"')";
	    		rs1.execute(sql);
	    		
	    		sql = "select id from workflow_FwAdvanced where uuid='"+uuid+"'";
	    		rs1.execute(sql);
	    		if(rs1.next()){
	    			mainid = rs1.getInt("id");
	    		}
				
	    		sql = "insert into workflow_FwOperAdvanced (mainid,objtype,objid,rolelevel,joblevel,jobobjid,relationship,seclevel,seclevelMax) ";
				sql += " select "+mainid+",objtype,objid,rolelevel,joblevel,jobobjid,relationship,seclevel,seclevelMax from workflow_FwOperAdvanced ";
				sql += " where mainid = "+currid+" order by id";
				rs1.execute(sql);
				
				sql = "insert into workflow_FwRecAdvanced (mainid,objtype,objid,rolelevel,joblevel,jobobjid,relationship,seclevel,seclevelMax) ";
				sql += " select "+mainid+",objtype,objid,rolelevel,joblevel,jobobjid,relationship,seclevel,seclevelMax from workflow_FwRecAdvanced ";
				sql += " where mainid = "+currid+" order by id";
				rs1.execute(sql);
			}
		}
		WorkflowAdvancedComInfo wfAdvancedComInfo = new WorkflowAdvancedComInfo();
		wfAdvancedComInfo.removeCache();
	}
	
	
	/**
	 * 普通模式同步
	 * @param param
	 * @param wfid
	 * @param nodeid
	 * @param fwtype 1转发 2意见征询 3转办
	 * @param user
	 * @param copy2nodeids  同步到目标节点 格式：1,2,3
	 */
	public void syncNodeSimple(Map<String, String> param,int wfid,int nodeid,String fwtype,User user,String copy2nodeids){
		String currentdate = TimeUtil.getCurrentDateString();
		String currenttime = TimeUtil.getOnlyCurrentTimeString();
		RecordSet rs = new RecordSet();
		RecordSet rs1 = new RecordSet();
		String sql = "";
		String sqlwhere = "";
		int id = Util.getIntValue(param.get("id"));
		String isopen = Util.null2String(param.get("isopen"));
		String modetype= Util.null2String(param.get("modetype"));
		String fromNewnode = Util.null2String(param.get("fromNewnode"));
		String choiceoperator = Util.null2String(param.get("choiceoperator"));
		
		if(!"".equals(copy2nodeids)){
			sqlwhere = " and nodeid in ("+copy2nodeids+") ";
		}else{
			sqlwhere = " and 1=2 ";
		}
		if(debug)writeLog(">>>>>>syncNodeSimple>>>>fromNewnode="+fromNewnode+" sqlwhere="+sqlwhere);		
		
		if(!"1".equals(fromNewnode)){ //新创建的节点 没有脏数据，不需要清除
			//接收人范围
			sql = "delete from workflow_FwRecSimple where mainid in (select id from workflow_FwLimitSet where fwtype='"+fwtype+"' "+sqlwhere+" and wfid="+wfid+")";
			rs.execute(sql);
		}
		
		if(debug)writeLog(">>>>>>syncNodeSimple>>>>sql="+sql);
		
		List<Integer> nodeids = new ArrayList<Integer>();
		List<String> oldnodeids = Util.splitString2List(copy2nodeids, ",");
		for (int i = 0; i < oldnodeids.size(); i++) {
			int tmpnodeid = Util.getIntValue(oldnodeids.get(i));
			if(tmpnodeid<=0)continue;
			nodeids.add(tmpnodeid);
		}
		
		if(debug)writeLog(">>>>>>syncNodeSimple>>>>nodeids="+nodeids);
		
		for (int k = 0; k < nodeids.size(); k++) {
			int tmpnodeid = nodeids.get(k);
			int mainid = 0;
			sql = "select id from workflow_FwLimitSet where fwtype='"+fwtype+"' and nodeid="+tmpnodeid+" and wfid="+wfid+" ";
			rs1.execute(sql);
			if(rs1.next()){
				mainid = rs1.getInt("id");
				sql = "update workflow_FwLimitSet set modetype=?,isopen=?,choiceoperator=?,lastoperator=?,lastoperatedate=?,lastoperatetime=? where id="+mainid;
				rs1.executeUpdate(sql, modetype,isopen,choiceoperator,user.getUID(),currentdate,currenttime);
			}else{
				sql = "insert into workflow_FwLimitSet(wfid,nodeid,fwtype,modetype,isopen,choiceoperator,lastoperator,lastoperatedate,lastoperatetime) values (?,?,?,?,?,?,?,?,?) ";
				rs1.executeUpdate(sql,wfid,tmpnodeid,fwtype, modetype,isopen,choiceoperator,user.getUID(),currentdate,currenttime);
			}
			
			if(mainid <= 0){
				sql = "select id from workflow_FwLimitSet where fwtype='"+fwtype+"' and nodeid="+tmpnodeid+" and wfid="+wfid+" ";
				rs1.execute(sql);
				if(rs1.next()){
					mainid = rs1.getInt("id");
				}
			}
			sql = "insert into workflow_FwRecSimple (mainid,objtype,objid,rolelevel,joblevel,jobobjid,relationship,seclevel,seclevelMax) ";
			sql += " select "+mainid+",objtype,objid,rolelevel,joblevel,jobobjid,relationship,seclevel,seclevelMax from workflow_FwRecSimple where mainid="+id+" order by id";
			rs1.execute(sql);
		}
		WorkflowRecSimpleComInfo wfRecSimpleComInfo = new WorkflowRecSimpleComInfo();
		wfRecSimpleComInfo.removeCache();
	}

	/**
	 * 保存基础信息，包含同步到所有节点
	 * @param user
	 * @param paramMap
	 * @return
	 */
	public Map<String,Object> saveFwLimitBaseSet(User user,Map<String,Object> paramMap){
		Map<String,Object> returnMap = new HashMap<String,Object>();
		int wfid = Util.getIntValue(Util.null2String(paramMap.get("wfid")),0);
		int nodeid = Util.getIntValue(Util.null2String(paramMap.get("nodeid")),0);
		String fwtype = Util.null2String(Util.null2String(paramMap.get("fwtype"))); // 1、转发，2、意见征询，3、转办
		int modetype = Util.getIntValue(Util.null2String(paramMap.get("modetype")),0); //1、高级模式，0或者其他 为普通模式
		String isopen = Util.null2o(Util.null2String( paramMap.get("isopen"))); // 1：开启 限制接收人可选范围
		int issynAll = Util.getIntValue(Util.null2String(paramMap.get("issynAll")),0);//1：同步到所有节点
		String choiceoperator = Util.null2o(Util.null2String(paramMap.get("choiceoperator")));

		int mainid = 0;
		String currentdate = TimeUtil.getCurrentDateString();
		String currenttime = TimeUtil.getOnlyCurrentTimeString();
		String sql = "";
		RecordSet rs = new RecordSet();
		sql = "select id from workflow_FwLimitSet where fwtype='"+fwtype+"' and nodeid="+nodeid+" and wfid="+wfid+" ";
		rs.execute(sql);
		if(rs.next()){
			mainid = rs.getInt("id");
			sql = "update workflow_FwLimitSet set modetype=?,isopen=?,choiceoperator=?,lastoperator=?,lastoperatedate=?,lastoperatetime=? where id="+mainid;
			rs.executeUpdate(sql, modetype,isopen,choiceoperator,user.getUID(),currentdate,currenttime);
		}else{
			sql = "insert into workflow_FwLimitSet(wfid,nodeid,fwtype,modetype,isopen,choiceoperator,lastoperator,lastoperatedate,lastoperatetime)";
			sql += " values ("+wfid+","+nodeid+",'"+fwtype+"','"+modetype+"','"+isopen+"','"+choiceoperator+"',"+user.getUID()+",'"+currentdate+"','"+currenttime+"') ";
			rs.execute(sql);

			sql = "select id from workflow_FwLimitSet where fwtype='"+fwtype+"' and nodeid="+nodeid+" and wfid="+wfid+" ";
			rs.execute(sql);
			if(rs.next()){
				mainid = rs.getInt("id");
			}
		}

		if(mainid>0 && issynAll==1){
			syncAllNode(wfid, nodeid, fwtype, user);
		}
		WorkflowRecSimpleComInfo wfRecSimpleComInfo = new WorkflowRecSimpleComInfo();
		wfRecSimpleComInfo.removeCache();

		WorkflowAdvancedComInfo wfAdvancedComInfo = new WorkflowAdvancedComInfo();
		wfAdvancedComInfo.removeCache();

		if(mainid>0){
			Map<String,Object> data = new HashMap<String,Object>();
			data.put("id",mainid);

			returnMap.put("success", "1");
			returnMap.put("msg", ""+ SystemEnv.getHtmlLabelName(18758,weaver.general.ThreadVarLanguage.getLang())+"");
			returnMap.put("hidden",data);

			paramMap.put("mainid",mainid);
			List<Object> conditions = new ArrayList<Object>();
			Map<String, Object> baseSetData = getBaseSetData(paramMap,user);
			if(baseSetData.containsKey("conditions")){
				conditions = (List<Object>)baseSetData.get("conditions");
			}
			if(baseSetData.containsKey("groupitems")){
				returnMap.put("groupitems",baseSetData.get("groupitems"));
			}
			if(baseSetData.containsKey("hidden")){
				Map<String,Object> hidden = (Map<String,Object>)baseSetData.get("hidden");
				data.putAll(hidden);
			}
			returnMap.put("hidden",data);
			returnMap.put("conditions",conditions);
		}else{
			returnMap.put("success", "0");
			returnMap.put("msg", ""+ SystemEnv.getHtmlLabelName(22620,weaver.general.ThreadVarLanguage.getLang())+"");
		}
		return returnMap;
	}

	/**
	 * 获取基础信息
	 * @param wfid
	 * @param nodeid
	 * @param fwtype
	 * @param user
	 * @return
	 */
	public Map<String,Object> getLimitBaseSet(int wfid,int nodeid,String fwtype,User user){
		Map<String,Object> returnMap = new HashMap<String,Object>();
        int mainid = 0;
        int modetype = 0;
        String isopen = "0";
        String choiceoperator = "0";

		String currentdate = TimeUtil.getCurrentDateString();
		String currenttime = TimeUtil.getOnlyCurrentTimeString();
		String sql = "";
		RecordSet rs = new RecordSet();
		sql = "select * from workflow_FwLimitSet where fwtype='"+fwtype+"' and nodeid="+nodeid+" and wfid="+wfid+" ";
		rs.execute(sql);
		if(rs.next()){
			mainid = rs.getInt("id");
			modetype = Util.getIntValue(rs.getString("modetype"));
			isopen = Util.null2o(rs.getString("isopen"));
			choiceoperator = Util.null2o(rs.getString("choiceoperator"));
		}else{
			sql = "insert into workflow_FwLimitSet(wfid,nodeid,fwtype,modetype,isopen,choiceoperator,lastoperator,lastoperatedate,lastoperatetime)";
			sql += " values ("+wfid+","+nodeid+",'"+fwtype+"','"+modetype+"','"+isopen+"','"+choiceoperator+"',"+user.getUID()+",'"+currentdate+"','"+currenttime+"') ";
			rs.execute(sql);

			sql = "select * from workflow_FwLimitSet where fwtype='"+fwtype+"' and nodeid="+nodeid+" and wfid="+wfid+" ";
			rs.execute(sql);
			if(rs.next()){
				mainid = rs.getInt("id");
				modetype = Util.getIntValue(rs.getString("modetype"));
				isopen = Util.null2o(rs.getString("isopen"));
				choiceoperator = Util.null2o(rs.getString("choiceoperator"));
			}
		}
		returnMap.put("mainid",mainid);
		returnMap.put("modetype",modetype);
		returnMap.put("isopen",isopen);
		returnMap.put("choiceoperator",choiceoperator);
		return returnMap;
	}


	/**
	 * 保存范围设置信息
	 * @param user
	 * @param paramMap
	 * @return
	 */
	public Map<String,Object> saveFwLimitSet(User user,Map<String,Object> paramMap){
		Map<String,Object> returnMap = new HashMap<String,Object>();

		int wfid = Util.getIntValue(Util.null2String(paramMap.get("wfid")),0);
		int nodeid = Util.getIntValue(Util.null2String(paramMap.get("nodeid")),0);
		String fwtype = Util.null2String(Util.null2String(paramMap.get("fwtype"))); // 1、转发，2、意见征询，3、转办

		int mainid = Util.getIntValue(Util.null2String(paramMap.get("mainid")),0);//普通模式 workflow_FwLimitSet的ID，高级模式 workflow_FwAdvanced的ID
		String modetype = Util.null2o(Util.null2String(paramMap.get("modetype"))); //1、高级模式，0或者其他 为普通模式
		int tabid = Util.getIntValue(Util.null2String(paramMap.get("tabid")),0); //高级模式--0、操作者  1、接收人范围

		int objtype=Util.getIntValue(Util.null2String(paramMap.get("objtype"))); //类型
		String relationship = Util.null2o(Util.null2String(paramMap.get("relationship")));  //接收人关系
		int seclevel = Util.getIntValue(Util.null2String(paramMap.get("seclevel")), 0);//最小安全级别
		int seclevelMax = Util.getIntValue(Util.null2String(paramMap.get("seclevelto")), 100); //最大安全级别
		//指定部门
		String departmentid = Util.null2String(paramMap.get("departmentid"));
		//指定分部
		String subids = Util.null2String(paramMap.get("subids"));
		//指定人员
		String userid = Util.null2String(paramMap.get("userid"));
		//指定角色
		String roleid = Util.null2String(paramMap.get("roleid"));
		int rolelevel=Util.getIntValue(Util.null2String( paramMap.get("rolelevel")));
		//指定岗位
		String jobid = Util.null2String(paramMap.get("jobid"));
		int joblevel=Util.getIntValue(Util.null2String( paramMap.get("joblevel")));
		String jobobjid = "";
		if(joblevel == 0){
		    jobobjid = Util.null2String(Util.null2String( paramMap.get("jobtitledepartment")));
		}else if(joblevel == 1){
		    jobobjid = Util.null2String(Util.null2String( paramMap.get("jobtitlesubcompany")));
		}

		RecordSet rs = new RecordSet();
		String sql = "";
		if("1".equals(modetype)) {//1、高级模式
			//没有数据，需要预先插入一条空的
			if(mainid<=0){
				String uuid = generateID();
				sql = "insert into workflow_FwAdvanced (wfid,nodeid,fwtype,uuid) values ("+wfid+","+nodeid+",'"+fwtype+"','"+uuid+"')";
				rs.execute(sql);

				sql = "select id from workflow_FwAdvanced where uuid='"+uuid+"'";
				rs.execute(sql);
				if(rs.next()){
					mainid = rs.getInt("id");
				}
			}
		}else{//0、普通模式
			sql = "select id from workflow_FwLimitSet where fwtype='"+fwtype+"' and nodeid="+nodeid+" and wfid="+wfid+" ";
			rs.execute(sql);
			if(rs.next()){
				mainid = rs.getInt("id");
			}
		}

		if(mainid<=0){
			returnMap.put("success", "0");
			returnMap.put("msg", ""+ SystemEnv.getHtmlLabelName(10004350,weaver.general.ThreadVarLanguage.getLang())+"mainid"+ SystemEnv.getHtmlLabelName(10004351,weaver.general.ThreadVarLanguage.getLang())+"");
			return returnMap;
		}

		if(objtype<=9 || objtype==15){
			if("1".equals(modetype)){//1、高级模式
				if(tabid == 1){//接收人范围
					sql = "insert into workflow_FwRecAdvanced (mainid,objtype,relationship,seclevel,seclevelMax) values (?,?,?,?,?) ";
					rs.executeUpdate(sql, mainid,objtype,relationship,seclevel,seclevelMax);
				}else{//操作者
					sql = "insert into workflow_FwOperAdvanced (mainid,objtype,seclevel,seclevelMax) values (?,?,?,?) ";
					rs.executeUpdate(sql, mainid,objtype,seclevel,seclevelMax);
				}
			}else{//0或者其他 为普通模式
				sql = "insert into workflow_FwRecSimple (mainid,objtype,relationship,seclevel,seclevelMax) values (?,?,?,?,?) ";
				rs.executeUpdate(sql, mainid,objtype,relationship,seclevel,seclevelMax);
			}

		}else if(objtype == 10||objtype==16){ //指定部门
			if("1".equals(modetype)){//1、高级模式
				if(tabid == 1){//接收人范围
					sql = "insert into workflow_FwRecAdvanced (mainid,objtype,objid,relationship,seclevel,seclevelMax) values (?,?,?,?,?,?) ";
					rs.executeUpdate(sql, mainid,objtype,departmentid,relationship,seclevel,seclevelMax);
				}else{//操作者
					sql = "insert into workflow_FwOperAdvanced (mainid,objtype,objid,seclevel,seclevelMax) values (?,?,?,?,?) ";
					rs.executeUpdate(sql, mainid,objtype,departmentid,seclevel,seclevelMax);
				}
			}else{//0或者其他 为普通模式
				sql = "insert into workflow_FwRecSimple (mainid,objtype,objid,relationship,seclevel,seclevelMax) values (?,?,?,?,?,?) ";
				rs.executeUpdate(sql, mainid,objtype,departmentid,relationship,seclevel,seclevelMax);
			}
		}else if(objtype == 11){ //指定分部
			if("1".equals(modetype)){//1、高级模式
				if(tabid == 1){//接收人范围
					sql = "insert into workflow_FwRecAdvanced (mainid,objtype,objid,relationship,seclevel,seclevelMax) values (?,?,?,?,?,?) ";
					rs.executeUpdate(sql, mainid,objtype,subids,relationship,seclevel,seclevelMax);
				}else{//操作者
					sql = "insert into workflow_FwOperAdvanced (mainid,objtype,objid,seclevel,seclevelMax) values (?,?,?,?,?) ";
					rs.executeUpdate(sql, mainid,objtype,subids,seclevel,seclevelMax);
				}
			}else{//0或者其他 为普通模式
				sql = "insert into workflow_FwRecSimple (mainid,objtype,objid,relationship,seclevel,seclevelMax) values (?,?,?,?,?,?) ";
				rs.executeUpdate(sql, mainid,objtype,subids,relationship,seclevel,seclevelMax);
			}
		}else if(objtype == 12){ //指定人员
			if("1".equals(modetype)){//1、高级模式
				if(tabid == 1){//接收人范围
					sql = "insert into workflow_FwRecAdvanced (mainid,objtype,objid,relationship) values (?,?,?,?) ";
					rs.executeUpdate(sql, mainid,objtype,userid,relationship);
				}else{//操作者
					sql = "insert into workflow_FwOperAdvanced (mainid,objtype,objid,relationship) values (?,?,?,?) ";
					rs.executeUpdate(sql, mainid,objtype,userid,relationship);
				}
			}else{//0或者其他 为普通模式
				sql = "insert into workflow_FwRecSimple (mainid,objtype,objid,relationship) values (?,?,?,?) ";
				rs.executeUpdate(sql, mainid,objtype,userid,relationship);
			}
		}else if(objtype == 13){ //指定角色
			if("1".equals(modetype)){//1、高级模式
				if(tabid == 1){//接收人范围
					sql = "insert into workflow_FwRecAdvanced (mainid,objtype,objid,rolelevel,relationship,seclevel,seclevelMax) values (?,?,?,?,?,?,?) ";
					rs.executeUpdate(sql, mainid,objtype,roleid,rolelevel,relationship,seclevel,seclevelMax);
				}else{//操作者
					sql = "insert into workflow_FwOperAdvanced (mainid,objtype,objid,rolelevel,seclevel,seclevelMax) values (?,?,?,?,?,?) ";
					rs.executeUpdate(sql, mainid,objtype,roleid,rolelevel,seclevel,seclevelMax);
				}
			}else{//0或者其他 为普通模式
				sql = "insert into workflow_FwRecSimple (mainid,objtype,objid,rolelevel,relationship,seclevel,seclevelMax) values (?,?,?,?,?,?,?) ";
				rs.executeUpdate(sql, mainid,objtype,roleid,rolelevel,relationship,seclevel,seclevelMax);
			}
		}else if(objtype == 14){ //指定岗位
			if("1".equals(modetype)){//1、高级模式
				if(tabid == 1){//接收人范围
					sql = "insert into workflow_FwRecAdvanced (mainid,objtype,objid,joblevel,jobobjid,relationship) values (?,?,?,?,?,?) ";
					rs.executeUpdate(sql, mainid,objtype,jobid,joblevel,jobobjid,relationship);
				}else{//操作者
					sql = "insert into workflow_FwOperAdvanced (mainid,objtype,objid,joblevel,jobobjid) values (?,?,?,?,?) ";
					rs.executeUpdate(sql, mainid,objtype,jobid,joblevel,jobobjid);
				}
			}else{//0或者其他 为普通模式
				sql = "insert into workflow_FwRecSimple (mainid,objtype,objid,joblevel,jobobjid,relationship) values (?,?,?,?,?,?) ";
				rs.executeUpdate(sql, mainid,objtype,jobid,joblevel,jobobjid,relationship);
			}
		}
		saveChangeLog(wfid, nodeid, fwtype, user);

		WorkflowRecSimpleComInfo wfRecSimpleComInfo = new WorkflowRecSimpleComInfo();
		wfRecSimpleComInfo.removeCache();

		WorkflowAdvancedComInfo wfAdvancedComInfo = new WorkflowAdvancedComInfo();
		wfAdvancedComInfo.removeCache();

        Map<String,Object> data = new HashMap<String,Object>();
		data.put("mainid",mainid);
		data.put("wfid",wfid);
		data.put("nodeid",nodeid);
		data.put("fwtype",fwtype);
		data.put("modetype",modetype);//1、高级模式，0或者其他 为普通模式
		data.put("tabid",tabid);//高级模式--0、操作者  1、接收人范围

		returnMap.put("success", "1");
		returnMap.put("msg", ""+ SystemEnv.getHtmlLabelName(18758,weaver.general.ThreadVarLanguage.getLang())+"");
		returnMap.put("hidden",data);

		paramMap.put("mainid",mainid);
		List<Object> conditions = new ArrayList<Object>();
		if("1".equals(modetype)) {//1、高级模式
			Map<String, Object> advanceData = getAdvanceData(paramMap,user);
			if(advanceData.containsKey("conditions")){
				conditions = (List<Object>)advanceData.get("conditions");
			}
		}else{
			Map<String, Object> baseSetData = getBaseSetData(paramMap,user);
			if(baseSetData.containsKey("conditions")){
				conditions = (List<Object>)baseSetData.get("conditions");
			}
			if(baseSetData.containsKey("groupitems")){
				returnMap.put("groupitems",baseSetData.get("groupitems"));
			}
			if(baseSetData.containsKey("hidden")){
				Map<String,Object> hidden = (Map<String,Object>)baseSetData.get("hidden");
				data.putAll(hidden);
			}
			returnMap.put("hidden",data);
		}
		returnMap.put("conditions",conditions);
		return returnMap;
	}

	/**
	 * 根据普通模式的接收人范围ID 、高级模式的接收人范围ID和操作者范围ID 做删除
	 * @param user 操作用户
	 * @param paramMap 其他参数
	 * @return
	 */
	public Map<String,Object> deleteFwLimitSet(User user,Map<String,Object> paramMap){
		Map<String,Object> returnMap = new HashMap<String,Object>();
		String ids = Util.null2String(Util.null2String(paramMap.get("ids"))); //接收人范围、操作者的ID，多个使用英文逗号隔开
		if(ids.startsWith(","))ids = ids.substring(1);
		if(ids.endsWith(","))ids = ids.substring(0,ids.length()-1);

		int wfid = Util.getIntValue(Util.null2String(paramMap.get("wfid")),0);
		int nodeid = Util.getIntValue(Util.null2String(paramMap.get("nodeid")),0);
		String fwtype = Util.null2String(Util.null2String(paramMap.get("fwtype"))); // 1、转发，2、意见征询，3、转办
		String modetype = Util.null2o(Util.null2String(paramMap.get("modetype"))); //1、高级模式，0或者其他 为普通模式

		//高级模式才需要的参数
		int mainid = Util.getIntValue(Util.null2String(paramMap.get("mainid")),0);
		int tabid = Util.getIntValue(Util.null2String(paramMap.get("tabid")),0); //高级模式--0、操作者  1、接收人范围

		if("".equals(ids)){
			returnMap.put("success", "0");
			returnMap.put("msg", ""+ SystemEnv.getHtmlLabelName(10004352,weaver.general.ThreadVarLanguage.getLang())+"ids"+ SystemEnv.getHtmlLabelName(18622,weaver.general.ThreadVarLanguage.getLang())+"");
			return returnMap;
		}

		RecordSet rs = new RecordSet();
		String sql = "";
		if("1".equals(modetype)){//1、高级模式
			if(tabid == 1){//接收人范围
				if(mainid<0){
					sql = "select mainid from workflow_FwRecAdvanced where id in ("+ids+") ";
					rs.execute(sql);
					if(rs.next()){
						mainid = rs.getInt("mainid");
					}
				}
				sql = "delete from workflow_FwRecAdvanced where id in ("+ids+") ";
				rs.execute(sql);
			}else{//操作者
				if(mainid<0){
					sql = "select mainid from workflow_FwOperAdvanced where id in ("+ids+") ";
					rs.execute(sql);
					if(rs.next()){
						mainid = rs.getInt("mainid");
					}
				}
				sql = "delete from workflow_FwOperAdvanced where id in ("+ids+") ";
				rs.execute(sql);
			}
			//同组操作者和接收人范围 都删干净了，则主表 一并删掉
			sql = "select id from workflow_FwRecAdvanced where mainid="+mainid+" union select id from workflow_FwOperAdvanced where mainid="+mainid;
			rs.execute(sql);
			if(!rs.next()){
				sql = "delete from workflow_FwAdvanced where id="+mainid;
				rs.execute(sql);
			}
		}else{//0或者其他 为普通模式
			sql = "delete from workflow_FwRecSimple where id in ("+ids+") ";
			rs.execute(sql);
		}
		saveChangeLog(wfid, nodeid, fwtype, user);

		WorkflowRecSimpleComInfo wfRecSimpleComInfo = new WorkflowRecSimpleComInfo();
		wfRecSimpleComInfo.removeCache();

		WorkflowAdvancedComInfo wfAdvancedComInfo = new WorkflowAdvancedComInfo();
		wfAdvancedComInfo.removeCache();

		Map<String,Object> data = new HashMap<String,Object>();
		data.put("mainid",mainid);
		data.put("wfid",wfid);
		data.put("nodeid",nodeid);
		data.put("fwtype",fwtype);
		data.put("modetype",modetype);//1、高级模式，0或者其他 为普通模式
		data.put("tabid",tabid);//高级模式--0、操作者  1、接收人范围

		returnMap.put("success", "1");
		returnMap.put("hidden",data);
		returnMap.put("msg", ""+ SystemEnv.getHtmlLabelName(20461,weaver.general.ThreadVarLanguage.getLang())+"");

		paramMap.put("mainid",mainid);
		List<Object> conditions = new ArrayList<Object>();
		if("1".equals(modetype)) {//1、高级模式
			Map<String, Object> advanceData = getAdvanceData(paramMap,user);
			if(advanceData.containsKey("conditions")){
				conditions = (List<Object>)advanceData.get("conditions");
			}
		}else{
			Map<String, Object> baseSetData = getBaseSetData(paramMap,user);
			if(baseSetData.containsKey("conditions")){
				conditions = (List<Object>)baseSetData.get("conditions");
			}
			if(baseSetData.containsKey("groupitems")){
				returnMap.put("groupitems",baseSetData.get("groupitems"));
			}
			if(baseSetData.containsKey("hidden")){
				Map<String,Object> hidden = (Map<String,Object>)baseSetData.get("hidden");
				data.putAll(hidden);
			}
			returnMap.put("hidden",data);
		}
		returnMap.put("conditions",conditions);
		return returnMap;
	}

	/**
	 * 高级模式，根据主表ID 删除对应的接收人范围、操作者范围以及主表
	 * @param user 操作用户
	 * @param paramMap 其他参数
	 * @return
	 */
	public Map<String,Object> deleteFwLimitSetByFwAdvanceId(User user,Map<String,Object> paramMap) {
		Map<String, Object> returnMap = new HashMap<String, Object>();
		String ids = Util.null2String(Util.null2String(paramMap.get("ids"))); //主表ID，多个使用英文逗号隔开
		if(ids.startsWith(","))ids = ids.substring(1);
		if(ids.endsWith(","))ids = ids.substring(0,ids.length()-1);

		int wfid = Util.getIntValue(Util.null2String(paramMap.get("wfid")),0);
		int nodeid = Util.getIntValue(Util.null2String(paramMap.get("nodeid")),0);
		String fwtype = Util.null2String(Util.null2String(paramMap.get("fwtype"))); // 1、转发，2、意见征询，3、转办
		String modetype = Util.null2o(Util.null2String(paramMap.get("modetype"))); //1、高级模式，0或者其他 为普通模式

		if("".equals(ids)){
			returnMap.put("success", "0");
			returnMap.put("msg", ""+ SystemEnv.getHtmlLabelName(10004352,weaver.general.ThreadVarLanguage.getLang())+"ids"+ SystemEnv.getHtmlLabelName(18622,weaver.general.ThreadVarLanguage.getLang())+"");
			return returnMap;
		}
		String sql = "";
		RecordSet rs = new RecordSet();
		//操作者
		sql = "delete from workflow_FwOperAdvanced where mainid in ("+ids+") ";
		rs.execute(sql);

		//接收人范围
		sql = "delete from workflow_FwRecAdvanced where mainid in ("+ids+") ";
		rs.execute(sql);

		//操作者与接收人范围 同组
		sql = "delete from workflow_FwAdvanced where id in ("+ids+") ";
		rs.execute(sql);

		saveChangeLog(wfid, nodeid, fwtype, user);

		WorkflowRecSimpleComInfo wfRecSimpleComInfo = new WorkflowRecSimpleComInfo();
		wfRecSimpleComInfo.removeCache();

		WorkflowAdvancedComInfo wfAdvancedComInfo = new WorkflowAdvancedComInfo();
		wfAdvancedComInfo.removeCache();

		Map<String,Object> data = new HashMap<String,Object>();
		data.put("wfid",wfid);
		data.put("nodeid",nodeid);
		data.put("fwtype",fwtype);
		data.put("modetype",modetype);//1、高级模式，0或者其他 为普通模式

		returnMap.put("success", "1");
		returnMap.put("msg", ""+ SystemEnv.getHtmlLabelName(20461,weaver.general.ThreadVarLanguage.getLang())+"");

		List<Object> conditions = new ArrayList<Object>();
		Map<String, Object> baseSetData = getBaseSetData(paramMap,user);
		if(baseSetData.containsKey("conditions")){
			conditions = (List<Object>)baseSetData.get("conditions");
		}
		if(baseSetData.containsKey("groupitems")){
			returnMap.put("groupitems",baseSetData.get("groupitems"));
		}
		if(baseSetData.containsKey("hidden")){
			Map<String,Object> hidden = (Map<String,Object>)baseSetData.get("hidden");
			data.putAll(hidden);
		}
		returnMap.put("hidden",data);
		returnMap.put("conditions",conditions);
		return returnMap;
	}
	
	/**
	 * 记录最后修改日期
	 * @param wfid
	 * @param nodeid
	 * @param fwtype
	 * @param user
	 */
	public void saveChangeLog(int wfid,int nodeid,String fwtype,User user){
		String currentdate = TimeUtil.getCurrentDateString();
		String currenttime = TimeUtil.getOnlyCurrentTimeString();
		RecordSet rs = new RecordSet();
		String sql = "update workflow_FwLimitSet set lastoperator=?,lastoperatedate=?,lastoperatetime=? where fwtype='"+fwtype+"' and nodeid="+nodeid+" and wfid="+wfid+" ";
		rs.executeUpdate(sql,user.getUID(),currentdate,currenttime);
	}


	public Map<String, Object> getBaseSetData(Map<String, Object> params,User user){
		Map<String, Object> apidatas = new HashMap<String,Object>();

		int wfid = Util.getIntValue(Util.null2String(params.get("wfid")),0);
		int nodeid = Util.getIntValue(Util.null2String(params.get("nodeid")),0);
		String fwtype = Util.null2String(Util.null2String(params.get("fwtype"))); // 1、转发，2、意见征询，3、转办
		String modetype = Util.null2String(Util.null2String(params.get("modetype")));
		int mainid = 0;
		String isopen = "0";
		String choiceoperator = "";
		RecordSet rs = new RecordSet();
		WfFwLimitTransMethod transMethod = new WfFwLimitTransMethod();
		Map<String,Object> baseMap = getLimitBaseSet(wfid,nodeid,fwtype,user);
		if("".equals(modetype)){
			modetype = Util.null2o(Util.null2String(baseMap.get("modetype")));
		}
		mainid = Util.getIntValue(Util.null2String(baseMap.get("mainid")));
		isopen = Util.null2o(Util.null2String(baseMap.get("isopen")));
		choiceoperator = Util.null2o(Util.null2String(baseMap.get("choiceoperator")));

		//供前台发送请求传递用
		Map<String,Object> hiddenMap = new HashMap<String,Object>();
		hiddenMap.put("wfid",wfid);
		hiddenMap.put("nodeid",nodeid);
		hiddenMap.put("fwtype",fwtype);
		hiddenMap.put("modetype",modetype);
		hiddenMap.put("isopen",isopen);
		hiddenMap.put("mainid",mainid);
		hiddenMap.put("modetypeTip", SystemEnv.getHtmlLabelName(386088, user.getLanguage()));
		hiddenMap.put("choiceoperator",choiceoperator);
		hiddenMap.put("choiceoperatorTip", SystemEnv.getHtmlLabelName(501043, user.getLanguage()));
		if("6".equals(fwtype)) {
			hiddenMap.put("choiceoperatorTip", SystemEnv.getHtmlLabelName(521995, user.getLanguage()));
		}

		List<Object> condition = new ArrayList<Object>();
		List<Object> condition2 = new ArrayList<Object>();
		Map<String,String> conditionMap = new HashMap<String,String>();
		if("1".equals(modetype)){//高级模式

			conditionMap.put("title", SystemEnv.getHtmlLabelName(99,user.getLanguage()));
			conditionMap.put("dataIndex","id1");
			conditionMap.put("key","id1");
			condition2.add(conditionMap);

			conditionMap = new HashMap<String,String>();
			conditionMap.put("title",SystemEnv.getHtmlLabelName(386137,user.getLanguage()));
			conditionMap.put("dataIndex","id2");
			conditionMap.put("key","id2");
			condition2.add(conditionMap);
			condition.add(condition2);

			condition2 = new ArrayList<Object>();
			String sql = "select * from workflow_FwAdvanced where fwtype='"+fwtype+"' and nodeid="+nodeid+" and wfid="+wfid+" ORDER BY id";
			rs.execute(sql);
			while (rs.next()){
				String id= rs.getString("id");
				String id_show = transMethod.getOperatorShow(id,user.getLanguage()+"");
				String id2_show = transMethod.getRecLimitShow(id,user.getLanguage()+"");

				conditionMap = new HashMap<String,String>();
				conditionMap.put("key",id);
				conditionMap.put("id1",id_show);
				conditionMap.put("id2",id2_show);
				condition2.add(conditionMap);
			}
			condition.add(condition2);
		}else{
			conditionMap = new HashMap<String,String>();
			conditionMap.put("title",SystemEnv.getHtmlLabelName(386135,user.getLanguage()));
			conditionMap.put("dataIndex","objtype");
			conditionMap.put("key","objtype");
			condition2.add(conditionMap);

			conditionMap = new HashMap<String,String>();
			conditionMap.put("title",SystemEnv.getHtmlLabelName(345,user.getLanguage()));
			conditionMap.put("dataIndex","objid");
			conditionMap.put("key","objid");
			condition2.add(conditionMap);

			conditionMap = new HashMap<String,String>();
			conditionMap.put("title",SystemEnv.getHtmlLabelName(683,user.getLanguage()));
			conditionMap.put("dataIndex","seclevel");
			conditionMap.put("key","seclevel");
			condition2.add(conditionMap);

			conditionMap = new HashMap<String,String>();
			conditionMap.put("title",SystemEnv.getHtmlLabelName(386136,user.getLanguage()));
			conditionMap.put("dataIndex","relationship");
			conditionMap.put("key","relationship");
			condition2.add(conditionMap);
			condition.add(condition2);

			condition2 = new ArrayList<Object>();
			String sql = "select * from workflow_FwRecSimple where mainid="+mainid+" ORDER BY id";
			rs.execute(sql);
			while (rs.next()){
				String id = rs.getString("id");
				String objtype = rs.getString("objtype");
				String objid = rs.getString("objid");
				String seclevel = rs.getString("seclevel");
				String seclevelMax = rs.getString("seclevelMax");
				String relationship = rs.getString("relationship");
				String rolelevel = rs.getString("rolelevel");
				String joblevel = rs.getString("joblevel");
				String jobobjid = rs.getString("jobobjid");

				String para = objtype+"+"+rolelevel+"+"+joblevel+"+"+jobobjid+"+"+user.getLanguage();
				String objtype_show = transMethod.getObjTypeShow(objtype,user.getLanguage()+"");
				String objid_show = transMethod.getObjIdsShow(objid,para);
				String seclevel_show = transMethod.getSecLevelShow(Util.getIntValue(objtype),seclevel,seclevelMax);
				String relationship_show=transMethod.getRelationshipShow(relationship,user.getLanguage()+"");

				conditionMap = new HashMap<String,String>();
				conditionMap.put("key",id);
				conditionMap.put("objtype",objtype_show);
				conditionMap.put("objid",objid_show);
				conditionMap.put("seclevel",seclevel_show);
				conditionMap.put("relationship",relationship_show);
				condition2.add(conditionMap);
			}
			condition.add(condition2);
		}

		ConditionFactory conditionFactory = new ConditionFactory(user);
		List<SearchConditionItem> items = new ArrayList<SearchConditionItem>();
		SearchConditionItem item = conditionFactory.createCondition(ConditionType.CHECKBOX,21738,"issynAll");
		item.setColSpan(8);
		item.setFieldcol(16);
		item.setLabelcol(8);
		if(!"6".equals(fwtype)) {
			items.add(item);
		}
		item = conditionFactory.createCondition(ConditionType.SWITCH,386133,"isopen");
		item.setValue(isopen);
		item.setColSpan(8);
		item.setFieldcol(16);
		item.setLabelcol(8);
		items.add(item);

		//转发 意见征询 传阅
		if("1".equals(fwtype) || "2".equals(fwtype)  || "4".equals(fwtype) || "6".equals(fwtype)){
			int _labelId = 500745;
			if("2".equals(fwtype))_labelId = 501045;
			if("4".equals(fwtype))_labelId = 504414;
			if("6".equals(fwtype))_labelId = 521994;
			item = conditionFactory.createCondition(ConditionType.SWITCH,_labelId,"choiceoperator");
			item.setValue(choiceoperator);
			item.setColSpan(8);
			item.setFieldcol(16);
			item.setLabelcol(8);
			items.add(item);
		}

		List<SearchConditionOption> options = new ArrayList<SearchConditionOption>();
		options.add(new SearchConditionOption("0",SystemEnv.getHtmlLabelName(18016, user.getLanguage()),"1".equals(modetype)?false:true));
		options.add(new SearchConditionOption("1",SystemEnv.getHtmlLabelName(19048, user.getLanguage()),"1".equals(modetype)?true:false));
		SearchConditionItem modetypeItem = conditionFactory.createCondition(ConditionType.SELECT,384810,"modetype",options);
		modetypeItem.setDetailtype(3); //3:radio单选
		modetypeItem.setColSpan(8);
		modetypeItem.setFieldcol(16);
		modetypeItem.setLabelcol(8);
		items.add(modetypeItem);

		apidatas.put("hidden",hiddenMap);
		apidatas.put("groupitems", new SearchConditionGroup(SystemEnv.getHtmlLabelName(387260, user.getLanguage()),true,items));
		apidatas.put("conditions", condition);
		apidatas.put("success", "1");
		return apidatas;
	}

	/**
	 * 获取高级模式下的  操作者和接收人范围列表
	 * @param params
	 * @param user
	 * @return
	 */
	public Map<String, Object> getAdvanceData(Map<String, Object> params,User user){
		Map<String, Object> apidatas = new HashMap<String,Object>();

		int wfid = Util.getIntValue(Util.null2String(params.get("wfid")),0);
		int nodeid = Util.getIntValue(Util.null2String(params.get("nodeid")),0);
		String fwtype = Util.null2String(Util.null2String(params.get("fwtype"))); // 1、转发，2、意见征询，3、转办

		int mainid = Util.getIntValue(Util.null2String(params.get("mainid"))); //workflow_FwAdvanced表的id

		List<Object> condition = new ArrayList<Object>();
		List<Object> condition2 = new ArrayList<Object>();
		Map<String,String> conditionMap = new HashMap<String,String>();
		WfFwLimitTransMethod transMethod = new WfFwLimitTransMethod();

		//供前台发送请求传递用
		Map<String,Object> hiddenMap = new HashMap<String,Object>();
		hiddenMap.put("wfid",wfid);
		hiddenMap.put("nodeid",nodeid);
		hiddenMap.put("fwtype",fwtype);
		hiddenMap.put("mainid",mainid);

		//操作者范围数据
		conditionMap = new HashMap<String,String>();
		conditionMap.put("title",SystemEnv.getHtmlLabelName(130470,user.getLanguage()));
		conditionMap.put("dataIndex","objtype");
		conditionMap.put("key","objtype");
		condition2.add(conditionMap);

		conditionMap = new HashMap<String,String>();
		conditionMap.put("title",SystemEnv.getHtmlLabelName(345,user.getLanguage()));
		conditionMap.put("dataIndex","objid");
		conditionMap.put("key","objid");
		condition2.add(conditionMap);

		conditionMap = new HashMap<String,String>();
		conditionMap.put("title",SystemEnv.getHtmlLabelName(683,user.getLanguage()));
		conditionMap.put("dataIndex","seclevel");
		conditionMap.put("key","seclevel");
		condition2.add(conditionMap);
		condition.add(condition2);

		condition2 = new ArrayList<Object>();
		String sql = "select * from workflow_FwOperAdvanced where mainid = "+mainid+" ORDER BY id";
		RecordSet rs = new RecordSet();
		rs.execute(sql);
		while(rs.next()){
			int id = rs.getInt("id");
			String objtype = rs.getString("objtype");
			String objid = rs.getString("objid");
			String rolelevel = rs.getString("rolelevel");
			String joblevel = rs.getString("joblevel");
			String jobobjid = rs.getString("jobobjid");
			String seclevel = rs.getString("seclevel");
			String seclevelMax = rs.getString("seclevelMax");

			String objtype_show = transMethod.getObjTypeShow(objtype,user.getLanguage()+"");
			String para = objtype+"+"+rolelevel+"+"+joblevel+"+"+jobobjid+"+"+user.getLanguage();
			String objid_show = transMethod.getObjIdsShow(objid,para);
			String seclevel_show = transMethod.getSecLevelShow(Util.getIntValue(objtype),seclevel,seclevelMax);

			conditionMap = new HashMap<String,String>();
			conditionMap.put("key",id+"");
			conditionMap.put("objtype",objtype_show);
			conditionMap.put("objid",objid_show);
			conditionMap.put("seclevel",seclevel_show);
			condition2.add(conditionMap);
		}
		condition.add(condition2);

		//接收人范围数据
		condition2 = new ArrayList<Object>();
		conditionMap = new HashMap<String,String>();
		conditionMap.put("title",SystemEnv.getHtmlLabelName(386135,user.getLanguage()));
		conditionMap.put("dataIndex","objtype");
		conditionMap.put("key","objtype");
		condition2.add(conditionMap);

		conditionMap = new HashMap<String,String>();
		conditionMap.put("title",SystemEnv.getHtmlLabelName(345,user.getLanguage()));
		conditionMap.put("dataIndex","objid");
		conditionMap.put("key","objid");
		condition2.add(conditionMap);

		conditionMap = new HashMap<String,String>();
		conditionMap.put("title",SystemEnv.getHtmlLabelName(683,user.getLanguage()));
		conditionMap.put("dataIndex","seclevel");
		conditionMap.put("key","seclevel");
		condition2.add(conditionMap);

		conditionMap = new HashMap<String,String>();
		conditionMap.put("title",SystemEnv.getHtmlLabelName(386136,user.getLanguage()));
		conditionMap.put("dataIndex","relationship");
		conditionMap.put("key","relationship");
		condition2.add(conditionMap);
		condition.add(condition2);

		condition2 = new ArrayList<Object>();
		sql = "select * from workflow_FwRecAdvanced where mainid = "+mainid+" ORDER BY id";
		rs.execute(sql);
		while(rs.next()){
			int id = rs.getInt("id");
			String objtype = rs.getString("objtype");
			String objid = rs.getString("objid");
			String rolelevel = rs.getString("rolelevel");
			String joblevel = rs.getString("joblevel");
			String jobobjid = rs.getString("jobobjid");
			String seclevel = rs.getString("seclevel");
			String seclevelMax = rs.getString("seclevelMax");
			String relationship = rs.getString("relationship");

			String objtype_show = transMethod.getObjTypeShow(objtype,user.getLanguage()+"");
			String para = objtype+"+"+rolelevel+"+"+joblevel+"+"+jobobjid+"+"+user.getLanguage();
			String objid_show = transMethod.getObjIdsShow(objid,para);
			String seclevel_show = transMethod.getSecLevelShow(Util.getIntValue(objtype),seclevel,seclevelMax);
			String relationship_show = transMethod.getRelationshipShow(relationship,user.getLanguage()+"");

			conditionMap = new HashMap<String,String>();
			conditionMap.put("key",id+"");
			conditionMap.put("objtype",objtype_show);
			conditionMap.put("objid",objid_show);
			conditionMap.put("seclevel",seclevel_show);
			conditionMap.put("relationship",relationship_show);
			condition2.add(conditionMap);
		}
		condition.add(condition2);
		apidatas.put("conditions", condition);
		apidatas.put("hidden",hiddenMap);
		apidatas.put("success", "1");
		return apidatas;
	}

	/**
	 * 允许选择节点参与人 作为 转发对象、意见征询对象
	 * @param params
	 * @param user
	 * @return
	 */
	public Map<String, Object> canChoiceOperator(Map<String, Object> params,User user){
		Map<String, Object> apidatas = new HashMap<String,Object>();
		int wfid = Util.getIntValue(Util.null2String(params.get("wfid")),0);
		int nodeid = Util.getIntValue(Util.null2String(params.get("nodeid")),0);
		String fwtype = Util.null2String(Util.null2String(params.get("fwtype"))); // 1、转发，2、意见征询，3、转办
		String choiceoperator = this.choiceOperator(wfid,nodeid,fwtype)?"1":"0";
		apidatas.put("choiceoperator", choiceoperator);
		apidatas.put("success", "1");
		return apidatas;
	}

}
